//env file
require('dotenv').config();
//Import require module
const http=require('http');
const https=require('https');
const createError = require('http-errors');
const express = require('express');
const bodyParser = require('body-parser');
const app=express();
const port=process.env.PORT;
const logger = require('morgan');
const path=require('path');
const fs=require('fs');
const cors = require('cors');


/* Routes import */
const routes=require('./routes/index.js');
/* support json encoded bodies */
app.use(bodyParser.json({ limit: '50mb' })); 
/* uploads use */
app.use('/uploads',express.static(path.join(__dirname,"uploads")));
/* support encoded bodies */
app.use(bodyParser.urlencoded({ limit: '50mb', extended: true })); 
/* produce a log entry */
app.use(logger('dev'));
//use cors
app.use(cors());
app.options('*', cors());
/* using middleware */
app.use(function(req, res, next) {
    res.setHeader('Access-Control-Allow-Origin', '*');
    res.setHeader('Access-Control-Allow-Methods', 'GET,PUT,POST,DELETE,PATCH,OPTIONS,HEAD');
    res.setHeader('Access-Control-Allow-Headers', 'Content-Type,access-token,email-token,x-xsrf-token,Tus-Resumable,Upload-Offset,Upload-Length,Upload-Metadata,Tus-Extension,x-http-method-override,x-requested-with');
    res.setHeader('Access-Control-Expose-Headers','Content-Type,expire');
    next();
});
/* using routes */
app.use(routes);
/*connecting to Database*/
require('./config/database');
//create server
if(process.env.IS_SSL=="true"){
    /*HTTPS *//* ssl configuration */
    const key  = fs.readFileSync('/home/devuipl/sslkey/Private');
    const cert = fs.readFileSync('/home/devuipl/sslkey/Certificate');
    const ca   = fs.readFileSync('/home/devuipl/sslkey/cabundle');
    const options = {
        key: key,
        cert: cert,
        ca: ca
    };
    var server = https.createServer(options,app);
}
else
    var server=http.createServer(app);
//starting server 
server.listen(port,err=>{
    if(err)
        throw err;
    else
        console.log(`server started on the port ${port}`);
})
module.exports = app;
