/* Importing schemas */
const FluencyComprehensionFormativeTestResult = require('./schemas/FluencyComprehensionFormativeTestResultSchema');
const FluencyComprehensionFormativeQuestionAnswer = require('./schemas/FluencyComprehensionFormativeQuestionAnswerSchema');
const FluencyComprehensionFormativeComplexityBand = require('./schemas/FluencyComprehensionFormativeComplexityBand');

const StudentTeacher = require('./schemas/StudentTeacherSchema');
/* importing */
const async=require('async');

/* Importing database*/
const Sequelize = require('sequelize');
const sequelize = require('../config/database').sequelize;
var fluency_comprehension_Formative_questions={};
//get Formative complexity bands
fluency_comprehension_Formative_questions.getFluencyComprehensionFormativeComplexityBands=(data)=>{
	return new Promise((resolve,reject)=>{
		FluencyComprehensionFormativeComplexityBand.findAll({
			where:{
				grade:data.grade,
				is_deleted:0,
				is_active:1,
			},
			attributes:['id','complexity_band'],
		}).then(response=>{
			var complexity_bands=JSON.parse(JSON.stringify(response));
			resolve({status:1,message:"All Formative Complexity bands",complexity_bands:complexity_bands});
		}).catch(err=>{
			reject({status:0,error:err,message:"Error in server. Please try after some time"});
		})
	});
}
//get Formative paragraph
fluency_comprehension_Formative_questions.getFluencyComprehensionFormativeParagraphs=(data)=>{
	return new Promise((resolve,reject)=>{
		async.waterfall([
			getTotalNumberParagraphComplexityBand,
			getLastTestResult,
			nextParagraphSequence
		],(err, result)=> {
			if(err.status==0) reject(result);
			else resolve(result);
		});
		//get total number of paragraph in complexity band and grade
		function getTotalNumberParagraphComplexityBand(callback){
			FluencyComprehensionFormativeQuestionAnswer.count({
				where:{
					grade:data.grade,
					complexity_band:data.complexity_band,
					is_deleted:0,
					is_active:1,
				},
			}).then(total_question=>{
				data.total_question=total_question;
				callback(null);
			}).catch(err=>{
				callback({status:0},{status:0,error:err,message:"Error in server. Please try after some time"});
			})
		}
		//get last test result paragraph sequence
		function getLastTestResult(callback){
			FluencyComprehensionFormativeTestResult.findOne({
				where:{
					student_grade_level:data.grade,
					complexity_band:data.complexity_band
				},
				order:[['test_result_id','desc']]
			}).then(test_result=>{
				test_result=JSON.parse(JSON.stringify(test_result));
				if(test_result){
					if(parseInt(test_result.paragraph_sequence) == data.total_question) data.paragraph_sequence=1;
					else data.paragraph_sequence = parseInt(test_result.paragraph_sequence) + 1;
				}else{
					data.paragraph_sequence=1;
				}
				callback(null);
			}).catch(err=>{
				callback({status:0},{status:0,error:err,message:"Error in server. Please try after some time"});
			})
		}
		//next Paragraph Sequence
		function nextParagraphSequence(callback){
			FluencyComprehensionFormativeQuestionAnswer.findOne({
				where:{
					grade:data.grade,
					complexity_band:data.complexity_band,
					is_deleted:0,
					paragraph_sequence:data.paragraph_sequence
				},
				attributes:['paragraph_id','paragraph_sequence','paragraph_text'],
			}).then(response=>{
				var formative_pargarphs=JSON.parse(JSON.stringify(response));
				callback({status:1},{status:1,message:"All Formative Pargraphs",formative_pargarphs:formative_pargarphs});
			}).catch(err=>{
				callback({status:0},{status:0,error:err,message:"Error in server. Please try after some time"});
			})
		}
	});
}
//get Formative question
fluency_comprehension_Formative_questions.getFluencyComprehensionFormativeQuestions=(data)=>{
	return new Promise((resolve,reject)=>{
		FluencyComprehensionFormativeQuestionAnswer.findOne({
			where:data,
			attributes:['formative_question_answer_id','question_no','question_text','correct_answer'],
            order:[['question_no','asc']]
		}).then(response=>{
			var formative_questions=JSON.parse(JSON.stringify(response));
			resolve({status:1,message:"All Formative questions",formative_questions:formative_questions});
		}).catch(err=>{
			reject({status:0,error:err,message:"Error in server. Please try after some time"});
		})
	});
}
//Save Fluency comprehension Test Result
fluency_comprehension_Formative_questions.saveFluencyComprehensionTestResult=(data)=>{
	return new Promise((resolve,reject)=>{
		async.waterfall([
			checkStudentExistence,
			checkParagraphExistence,
			saveFluencyComprehensionTestResult,
		],(err, result)=> {
			if(err.status==0) reject(result);
			else resolve(result);
		});
		//check student exist or not using student teachers id and grade level
		function checkStudentExistence(callback){
			StudentTeacher.findOne({
				where:{ 
					student_teachers_id:data.student_teachers_id,	
					student_grade_level:data.student_grade_level 
				},
			}).then(student_teachers_details=>{
				if(student_teachers_details) callback(null,student_teachers_details);
				else callback({status:1},{status:0,message:"Invalid Student!"});
			}).catch(err=>{
				callback({status:0},{status:0,error:err,message:"Error in server. Please try after some time"});
			})
		}
		//check paragraph id
		function checkParagraphExistence(student_teachers_details,callback){
			FluencyComprehensionFormativeQuestionAnswer.findOne({
				where:{
					grade:data.student_grade_level,
					is_deleted:0,
					is_active:1,
					paragraph_id:data.paragraph_id,
					paragraph_sequence:data.paragraph_sequence
				}	
			}).then(paragraphs=>{
				paragraphs=JSON.parse(JSON.stringify(paragraphs));
				if(paragraphs){
					data.complexity_band=paragraphs.complexity_band;
					callback(null,student_teachers_details);
				}
				else callback({status:1},{status:0,message:"Invalid Pargraph id or squence number!"});
			}).catch(err=>{
				callback({status:0},{status:0,error:err,message:"Error in server. Please try after some time"});
			})
		}
        //Save Fluency comprehension Test Result
		function saveFluencyComprehensionTestResult(student_teachers_details,callback){

			var student_teachers_details = JSON.parse(JSON.stringify(student_teachers_details));
			console.log(student_teachers_details)

			data.student_id=student_teachers_details.student_id;
			data.student_teachers_email_address=student_teachers_details.student_teachers_email_address;
			data.student_name=student_teachers_details.student_first_name+" "+student_teachers_details.student_middle_name+" "+student_teachers_details.student_last_name;
			data.student_teachers_name=student_teachers_details.student_teachers_first_name+" "+student_teachers_details.student_teachers_last_name;
			
			console.log(data)


			FluencyComprehensionFormativeTestResult.create(data).then(response=>{
				callback({status:1},{status:1,message:"Fluency comprehension test result for formative has been saved successfully"});
			}).catch(err=>{
				callback({status:0},{status:0,error:err,message:"Error in server. Please try after some time"});
			})
		}
	});
}
module.exports = fluency_comprehension_Formative_questions;


