/* Importing schemas */
const FluencyComprehensionSummativeParagraphQuestion = require('./schemas/FluencyComprehensionSummativeParagraphQuestionSchema');
const FluencyComprehensionSummativeTestResult = require('./schemas/FluencyComprehensionSummativeTestResultSchema');
const FluencyComprehensionSummativeQuestionAnswer = require('./schemas/FluencyComprehensionSummativeQuestionAnswerSchema');

const StudentTeacher = require('./schemas/StudentTeacherSchema');
/* importing */
const async=require('async');

/* Importing database*/
const Sequelize = require('sequelize');
const sequelize = require('../config/database').sequelize;
var fluency_comprehension_summative_questions={};
//get summative paragraph
fluency_comprehension_summative_questions.getFluencyComprehensionSummativeParagraphs=(data)=>{
	return new Promise((resolve,reject)=>{
		FluencyComprehensionSummativeParagraphQuestion.findAndCountAll({
			where:{
				grade:data.grade,
				session:data.session,
				is_deleted:0,
				is_active:1,
			},
			attributes:['paragraph_id','paragraph_sequence','paragraph_text'],
			order:[['paragraph_sequence','asc']],
			limit:data.limit,
			offset:data.offset
		}).then(response=>{
			var summative_pargarphs=JSON.parse(JSON.stringify(response));
			resolve({status:1,message:"All summative Pargraphs",total_paragraph_count:summative_pargarphs.count,summative_pargarphs:summative_pargarphs.rows});
		}).catch(err=>{
			reject({status:0,error:err,message:"Error in server. Please try after some time"});
		})
	});
}
//get summative question
fluency_comprehension_summative_questions.getFluencyComprehensionSummativeQuestions=(data)=>{
	return new Promise((resolve,reject)=>{
		FluencyComprehensionSummativeQuestionAnswer.findAll({
			where:data,
			attributes:['summative_question_id','question_no','question_text','option1','option2','option3','option4'],
            order:[['question_no','asc']]
		}).then(response=>{
			var summative_questions=JSON.parse(JSON.stringify(response));
			resolve({status:1,message:"All summative questions",summative_questions:summative_questions});
		}).catch(err=>{
			reject({status:0,error:err,message:"Error in server. Please try after some time"});
		})
	});
}
//Save Fluency comprehension Test Result
fluency_comprehension_summative_questions.saveFluencyComprehensionTestResult=(data)=>{
	return new Promise((resolve,reject)=>{
		async.waterfall([
			checkStudentExistence,
			checkParagraphExistence,
			correctQuestionAnswer,
			saveFluencyComprehensionTestResult,
		],(err, result)=> {
			if(err.status==0) reject(result);
			else resolve(result);
		});
		//check student exist or not using student teachers id and grade level
		function checkStudentExistence(callback){
			StudentTeacher.findOne({
				where:{ 
					student_teachers_id:data.student_teachers_id,	
					student_grade_level:data.student_grade_level 
				},
			}).then(student_teachers_details=>{
				if(student_teachers_details) callback(null,student_teachers_details);
				else callback({status:1},{status:0,message:"Invalid Student!"});
			}).catch(err=>{
				callback({status:0},{status:0,error:err,message:"Error in server. Please try after some time"});
			})
		}
		//check paragraph id
		function checkParagraphExistence(student_teachers_details,callback){
			FluencyComprehensionSummativeParagraphQuestion.count({
				where:{
					grade:data.student_grade_level,
					session:data.session,
					is_deleted:0,
					is_active:1,
					paragraph_id:data.paragraph_id,
					paragraph_sequence:data.paragraph_sequence
				}	
			}).then(paragraphs=>{
				if(paragraphs) callback(null,student_teachers_details);
				else callback({status:1},{status:0,message:"Invalid Pargraph id or squence number!"});
			}).catch(err=>{
				callback({status:0},{status:0,error:err,message:"Error in server. Please try after some time"});
			})
		}
		function correctQuestionAnswer(student_teachers_details,callback){
			var question_answers_arr=[];
			async.eachOfSeries(data.question_answers, function(question_answers,key,loopCallback) {
				FluencyComprehensionSummativeQuestionAnswer.findOne({ 
					where:{ 
						grade:data.student_grade_level,
						session:data.session,
						is_deleted:0,
						is_active:1,
						paragraph_id:data.paragraph_id,
						question_no:question_answers.question_no
					} 
				}).then(response=>{
					response=JSON.parse(JSON.stringify(response));
					if(response){
						question_answers_arr.push({question_no:question_answers.question_no,answer:question_answers.answer,correct_answer:response.correct_answer})
						if(question_answers.answer==response.correct_answer){
							data.count_correct_answer++;
						}
						loopCallback();
					}else{
						callback({status:0},{status:0,message:"Invalid question number!"});
					}
				}).catch(err=>{
					callback({status:0},{status:0,error:err,message:"Error in server. Please try after some time"});
				})
				
			},(err)=>{
				callback(null,student_teachers_details,question_answers_arr);
			});
		}
        //Save Fluency comprehension Test Result
		function saveFluencyComprehensionTestResult(student_teachers_details,question_answers_arr,callback){
			data.question_answers=JSON.stringify(question_answers_arr);
			data.student_id=student_teachers_details.student_id;
			data.student_teachers_email_address=student_teachers_details.student_teachers_email_address;
			data.student_name=student_teachers_details.student_first_name+" "+student_teachers_details.student_middle_name+" "+student_teachers_details.student_last_name;
			data.student_teachers_name=student_teachers_details.student_teachers_first_name+" "+student_teachers_details.student_teachers_last_name;
			FluencyComprehensionSummativeTestResult.create(data).then(response=>{
				callback({status:1},{status:1,message:"Fluency comprehension test result for summative has been saved successfully"});
			}).catch(err=>{
				callback({status:0},{status:0,error:err,message:"Error in server. Please try after some time"});
			})
		}
	});
}
module.exports = fluency_comprehension_summative_questions;
