/* Importing schemas */
const SpellingSummativeWord = require('./schemas/SpellingSummativeWordSchema');
const StudentTeacher = require('./schemas/StudentTeacherSchema');
const SpellingSummativeTestResult = require('./schemas/SpellingSummativeTestResultSchema');
/* importing */
const async = require('async');

/* Importing database*/
const Sequelize = require('sequelize');
const sequelize = require('../config/database').sequelize;
var spelling_summative_words = {};
//get spelling summative words
spelling_summative_words.getSpellingSummativeWord = (data, limit) => {
    return new Promise((resolve, reject) => {
        async.waterfall([
            getTotalNumberStage,
            getWordStageWise,
        ], (err, result) => {
            if (err.status == 0) reject(result);
            else resolve(result);
        });

        //get total number of stage
        function getTotalNumberStage(callback) {
            SpellingSummativeWord.count({
                group: ['category']
            }).then(total_word_count => {
                callback(null, total_word_count.length);
            }).catch(err => {
                callback({ status: 0 }, { status: 0, error: err, message: "Error in server. Please try after some time" });
            })
        }
        function getWordStageWise(total_word_count, callback) {
            SpellingSummativeWord.findAll({
                attributes: ['spelling_summative_word_id', 'word', 'audio'],
                where: data,
                limit: limit,
                order: [
                    [Sequelize.fn('RAND')]
                ]
            }).then(response => {
                var spelling_summative_words = JSON.parse(JSON.stringify(response));
                callback({ status: 1 }, { status: 1, message: "Spelling Summative Words", total_word_count: total_word_count, spelling_summative_words: spelling_summative_words });
            }).catch(err => {
                callback({ status: 0 }, { status: 0, error: err, message: "Error in server. Please try after some time" });
            })
        }

    });
}
//save spelling summative test result
spelling_summative_words.saveSpellingSummativeTestResult = (data) => {

    console.log("===============Test data=================")
    console.log(data);
    console.log("===============Test data=================")



    return new Promise((resolve, reject) => {
        async.waterfall([
            checkStudentExistence,
            checkWordExistence,
            checkMisspells1,
            saveSpellingSummativeTestResult,
            checkMisspells2
        ], (err, result) => {
            if (err.status == 0) reject(result);
            else resolve(result);
        });
        //check student exist or not using student teachers id and grade level
        function checkStudentExistence(callback) {
            StudentTeacher.findOne({
                where: {
                    student_teachers_id: data.student_teachers_id,
                    student_grade_level: data.student_grade_level
                },
            }).then(student_teachers_details => {
                if (student_teachers_details) callback(null, student_teachers_details);
                else callback({ status: 1 }, { status: 0, message: "Invalid Student!" });
            }).catch(err => {
                callback({ status: 0 }, { status: 0, error: err, message: "Error in server. Please try after some time" });
            })
        }
        //check speling word id
        function checkWordExistence(student_teachers_details, callback) {
            SpellingSummativeWord.findOne({
                where: {
                    spelling_summative_word_id: data.spelling_summative_word_id,
                    is_deleted: 0,
                    is_active: 1,
                }
            }).then(spelling_summative_word => {
                spelling_summative_word = JSON.parse(JSON.stringify(spelling_summative_word));
                if (spelling_summative_word) {
                    data.word = spelling_summative_word.word;
                    if (data.word.toLowerCase() == data.student_answer.toLowerCase()) data.is_correct_answer = 1;
                    else data.is_correct_answer = 0;
                    callback(null, student_teachers_details);
                }
                else callback({ status: 1 }, { status: 0, message: "Invalid speling summative word id!" });
            }).catch(err => {
                console.log(err);
                callback({ status: 0 }, { status: 0, error: err, message: "Error in server. Please try after some time" });
            })
        }
        function checkMisspells1(student_teachers_details, callback) {
            SpellingSummativeTestResult.findAll({
                where: {
                    student_teachers_id: data.student_teachers_id,
                    is_current_test: 1,
                    test_name: data.test_name
                },
                attributes: ['is_correct_answer'],
                limit: 5,
                order: [['test_result_id', 'desc']]

            }).then(test_results => {
                test_results = JSON.parse(JSON.stringify(test_results));
                let total_incorrect_word = 0;
                test_results.map(test_result => {
                    if (!test_result.is_correct_answer)
                        total_incorrect_word++;
                })                

                if (total_incorrect_word >= 4) {
                    //callback({ status: 1 }, { status: 1, message: "Spelling test has been ended due to misspells 4 of the last 5 words", is_discard_test: 1 });
                    callback({ status: 1 }, { status: 1, message: "Spelling assessment is completed", is_discard_test: 1 });
                
                } else {
                    callback(null, student_teachers_details);
                }
            }).catch(err => {
                callback({ status: 0 }, { status: 0, error: err, message: "Error in server. Please try after some time" });
            })
        }
        //save spelling summative test result
        function saveSpellingSummativeTestResult(student_teachers_details, callback) {
            data.student_id = student_teachers_details.student_id;
            data.student_teachers_email_address = student_teachers_details.student_teachers_email_address;
            data.student_name = student_teachers_details.student_first_name + " " + student_teachers_details.student_middle_name + " " + student_teachers_details.student_last_name;
            data.student_teachers_name = student_teachers_details.student_teachers_first_name + " " + student_teachers_details.student_teachers_last_name;
            //data.test_name = 

            SpellingSummativeTestResult.create(data).then(response => {
                callback(null);
            }).catch(err => {
                callback({ status: 0 }, { status: 0, error: err, message: "Error in server. Please try after some time" });
            })
        }
        //check 4 misspells out of 5
        function checkMisspells2(callback) {
            SpellingSummativeTestResult.findAll({
                where: {
                    student_teachers_id: data.student_teachers_id,
                    is_current_test: 1,
                    test_name: data.test_name
                },
                attributes: ['is_correct_answer'],
                limit: 5,
                order: [['test_result_id', 'desc']]

            }).then(test_results => {
                test_results = JSON.parse(JSON.stringify(test_results));
                let total_incorrect_word = 0;
                test_results.map(test_result => {
                    if (!test_result.is_correct_answer)
                        total_incorrect_word++;
                })


                console.log("===============Test Counter=================")
                console.log(total_incorrect_word);


                
                if (total_incorrect_word >= 4) {
                    //callback({ status: 1 }, { status: 1, message: "Spelling test has been ended due to misspells 4 of the last 5 words", is_discard_test: 1 });
                    callback({ status: 1 }, { status: 1, message: "Spelling assessment is completed", is_discard_test: 1 });
                
                } else {
                    callback({ status: 1 }, { status: 1, message: "Spelling test result for formative has been saved successfully", is_discard_test: 0 });
                }
            }).catch(err => {
                callback({ status: 0 }, { status: 0, error: err, message: "Error in server. Please try after some time" });
            })
        }
    });
}
//complete current spelling test
spelling_summative_words.completeCurrentSpellingTest = (student_teachers_id) => {
    return new Promise((resolve, reject) => {
        SpellingSummativeTestResult.update({ is_current_test: 0 }, {
            where: {
                student_teachers_id: student_teachers_id
            }
        }).then(res => {
            resolve({ status: 1, message: "Current spelling test has been completed successfully" });
        }).catch(err => {
            reject({ status: 0, error: err, message: "Error in server. Please try after some time" });
        })
    })
}
module.exports = spelling_summative_words;
