/* Importing schemas */
const StudentTeacher = require('./schemas/StudentTeacherSchema');
const StudentTeacherEmailToken = require('./schemas/StudentTeacherEmailTokenSchema');
const Token = require('./schemas/TokenSchema');
const Sentence = require('./schemas/SentenceSchema');
/* Importing database*/
const Sequelize = require('sequelize');
const sequelize = require('../config/database').sequelize;
/* importing */
const async=require('async');
const path=require('path');
const siteUrl = require('../config/siteUrl');
const helpers = require('../helpers/helper');
/* importing handlebars for replace html value name */
const handlebars=require('handlebars');

const moment=require('moment');
//Relationships
Token.belongsTo(StudentTeacher,{foreignKey: 'student_teachers_id',sourceKey:'student_teachers_id',as:'student_teacher_details'});
var student_teachers = {};
//check access token
student_teachers.checkAccessToken=function(data){
	return new Promise(function(resolve,reject){
		Token.findOne({ 
			where: data,
			attributes:['token_id','student_teachers_id','unique_token'],	
			include: [{ 
				model: StudentTeacher,
				attributes:['student_teachers_id','student_teachers_email_address','student_teachers_password','is_active','is_deleted'],
				as:'student_teacher_details'
			}]
		}).then(response=>{
			resolve(response);
		}).catch(err=>{
			reject({status:0,error:err,message:"Error in server.please do after some time"});
		})
	});
}
//Teachers registration
student_teachers.teachersRegistration=(data)=>{
	return new Promise((resolve,reject)=>{
		async.waterfall([
			checkTeacherExistence,
			teachersRegistration,
		],(err, result)=> {
			if(err.status==0) reject(result);
			else resolve(result);
		});
		//check teacher exist or not using email id and grade level
		function checkTeacherExistence(callback){
			var where={ 
				student_teachers_email_address:data.student_teachers_email_address,	
				student_grade_level:{$in:data.student_grade_level} 
			}
			StudentTeacher.findOne({
				where:where,
				attributes:['student_teachers_id','student_teachers_email_address','student_teachers_password']
			}).then(response=>{
				if(!response) callback({status:1},{status:0,message:"Please enter valid email id!"});
				else {
					if(response.student_teachers_password!=null && response.student_teachers_password!="") callback({status:1},{status:0,message:"You already completed your registration!"});
					else callback(null);
				}
			}).catch(err=>{
				callback({status:0},{status:0,error:err,message:"Error in server. Please try after some time"});
			})
		}
		function teachersRegistration(callback){
			var where={ 
				student_teachers_email_address:data.student_teachers_email_address,	
				student_grade_level:{$ne:''}
			}
			StudentTeacher.update({
					student_teachers_password:data.student_teachers_password,
				},{
				where:where
			}).then(response=>{
				callback({status:1},{status:1,message:"Your registration has been completed successfully",user_account_id:response.user_account_id});
			}).catch(err=>{
				callback({status:0},{status:0,error:err,message:"Error in server. Please try after some time"});
			})
		}
	});
}
//fetch all email id for grade level
student_teachers.fetchAllTeachersEmail=(data)=>{
	return new Promise((resolve,reject)=>{
		var where={ 
				student_grade_level:{$in:data.student_grade_level} 
		}
		StudentTeacher.findAll({
			where:where,
			attributes: [[sequelize.fn('DISTINCT', sequelize.col('student_teachers_email_address')), 'student_teachers_email_address']],
			distinct: true,
		}).then(response=>{
			resolve({status:1,message:"All Teachers Email List",result:response});
		}).catch(err=>{
			reject({status:0,error:err,message:"Error in server. Please try after some time"});
		})
	});
}
//Teachers login
student_teachers.teachersLogin=(data)=>{
	return new Promise((resolve,reject)=>{
		async.waterfall([
			checkTeacherExistence,
			insertToken,
		],(err, result)=> {
			if(err.status==0) reject(result);
			else resolve(result);
		});
		//check teacher exist or not using email id and grade level
		function checkTeacherExistence(callback){
			var where={ 
				student_teachers_email_address:data.student_teachers_email_address,	
				$and:[
					{student_grade_level:{$ne:''}},
					{student_grade_level:{ $in:data.student_grade_level} }
				],
				is_active:1,
				is_deleted:0
			}
			StudentTeacher.findOne({
				where:where,
				attributes:['student_teachers_id','student_teachers_first_name','student_teachers_last_name','student_teachers_email_address','student_teachers_password','student_grade_level'],
			}).then(response=>{
				if(!response) callback({status:1},{status:0,message:"This email id is not valid for this app!"});
				else {
					if(response.student_teachers_password!=null && response.student_teachers_password!=""){
						if(helpers.decrypt(response.student_teachers_password)==data.student_teachers_password) callback(null,response);
						else callback({status:1},{status:0,message:"Please enter a valid password!"});
					}else callback({status:1},{status:0,message:"Please complete your registration!"});
				}
			}).catch(err=>{
				callback({status:0},{status:0,error:err,message:"Error in server. Please try after some time"});
			})
		}
		//insert token
		function insertToken(teacher_details,callback) {
			var insert_token={
				student_teachers_id:teacher_details.student_teachers_id,
				unique_token:helpers.generateUniqueToken()
			};
			var teacher_details_response=JSON.parse(JSON.stringify(teacher_details));
			delete teacher_details_response.student_teachers_password;
			teacher_details_response.unique_token=insert_token.unique_token;
			Token.create(insert_token).then(response=>{
				callback({status:0},{status:1,message:"You have been successfully logged in...",teacher_details:teacher_details_response});
            }).catch(err=>{
                callback({status:1},{status:0,error:err,message:"Error in server. Please try again after some timeome time"});
            })
		}
	});
}
//logout
student_teachers.logout=(data)=>{
	return new Promise((resolve,reject)=>{
		Token.destroy({
			where:data
		}).then(response=>{
			resolve(response);
		}).catch(err=>{
			reject({status:0,error:err,message:"Error in server. Please try after some time"});
		})
	});
}
//forgot password
student_teachers.forgotPassword=(data,is_phone)=>{
	return new Promise((resolve,reject)=>{
        async.waterfall([
			checkTeacherExistence,
			generateTokenForForgotPassword,
			sentMail
        ], (err, result)=> {
            if(err.status == 1) reject(result);
            else resolve(result);
		});
		//check teacher existence
        function checkTeacherExistence(callback) {
			var where={ 
				student_teachers_email_address:data.student_teachers_email_address,
				student_teachers_password:{ $ne: "" },
				$and:{
					student_teachers_password:{ $ne: null }
				},
				$and:[
					{student_grade_level:{$ne:''}},
					{student_grade_level:{ $in:data.student_grade_level} }
				]
			}
            StudentTeacher.findOne({ 
					where:where,
					attributes : ['student_teachers_id','student_teachers_first_name','student_teachers_last_name','student_teachers_email_address','student_grade_level'],
				 }).then(teacher_details=>{
					if(teacher_details){
						callback(null,teacher_details);
					}
					else callback({status:0},{status:0,message:"Check & enter a valid registered email address before reset the password!"});
			}).catch(err=>{
                callback({status:1},{status:0,error:err,message:"Error in server. Please try again after some timeome time"});
           	})
		}
		//create forgot email token for forgot password
        function generateTokenForForgotPassword(teacher_details,callback) {
			var data={
				student_teachers_id:teacher_details.student_teachers_id,
				email:teacher_details.student_teachers_email_address,
				unique_key:helpers.generateUniqueToken(),
				token_type:"forgot_password"
			}
			StudentTeacherEmailToken.create(data).then(generate_token_details=>{
				callback(null,teacher_details,generate_token_details);
            }).catch(err=>{
                callback({status:1},{status:0,error:err,message:"Error in server. Please try again after some timeome time"});
            })
		}
		//sent mail
		function sentMail(teacher_details,generate_token_details,callback) {
			helpers.readHTMLFile(path.join(__dirname,'../email_templates/reset-password.html'), (err, html) =>{
				if(err) callback({status:0},{status:0,message:"Sorry! We are unable to sent mail to your email address: "+generate_token_details.email});
				else{
					if(data.student_grade_level.length==1){
						var FRONT_URL="http://www.letterid.tk/";
					}else if(data.student_grade_level.length==2){
						var FRONT_URL="http://sentence.letterid.tk/";
					}else{
						var FRONT_URL="http://fluency.letterid.tk/";
					}
					var template = handlebars.compile(html);
					var replacements = {
						full_name: teacher_details.student_teachers_first_name+' '+teacher_details.student_teachers_last_name,
						url:FRONT_URL+'reset-password/'+generate_token_details.email+'/'+generate_token_details.unique_key,
						support_email:siteUrl.support_email,
						site_name:siteUrl.site_name,
						mail_signature_by:siteUrl.mail_signature_by,
					};
					var html = template(replacements);
					helpers.sentMail(generate_token_details.email,"Reset Password",siteUrl.support_email,html,(response)=>{
						if(response.status==1){
							 callback({status:1},{status:1,message:"An email has been sent to your registered email - "+generate_token_details.email+". Please reset your password using reset password link available on your email!"});
						}else{
							callback({status:0},{status:0,message:"Sorry! We are unable to sent mail to your email address: "+generate_token_details.email});
						}
					});
				}
			})
		}
	});
}
//verify token for forgot password
student_teachers.verifyTokenForgotPassword=(data)=>{
	return new Promise((resolve,reject)=>{
        async.waterfall([
			checkTeacherExistence,
			verifyLatestToken,
			verifyToken,
			destroyAllToken
        ], (err, result)=> {
			if(err.status == 1) reject(result);
			else resolve(result);
		});
		//check teacher existence
	   function checkTeacherExistence(callback) {
			StudentTeacher.findOne({ 
					where:{ student_teachers_email_address:data.email },
					attributes : ['student_teachers_id','student_teachers_first_name','student_teachers_last_name','student_teachers_email_address'],
				}).then(teacher_details=>{
					if(teacher_details) callback(null,teacher_details);
					else callback({status:0},{status:0,message:"Check & enter a valid registered email address!"});
			}).catch(err=>{
				callback({status:1},{status:0,error:err,message:"Error in server. Please try again after some timeome time"});
			})
		}
		//get latest token
		function verifyLatestToken(teacher_details,callback) {
			StudentTeacherEmailToken.findOne({ 
				where:{email:teacher_details.student_teachers_email_address,token_type:"forgot_password" },
				order: [ ['email_token_id', 'DESC'] ]
			}).then(response=>{
				if(response){
					if(response.unique_key==data.unique_key) callback(null,teacher_details);
					else callback({status:0},{status:0,message:"Reset Password Link has been expired!"});
				}else callback({status:0},{status:0,message:"Invalid Reset Password Link!"});
					 
            }).catch(err=>{
                callback({status:1},{status:0,error:err,message:"Error in server. Please try again after some timeome time"});
            })
		}
		//verify token
        function verifyToken(teacher_details,callback) {
			StudentTeacherEmailToken.findOne({ 
				where:{
					email:teacher_details.student_teachers_email_address,
					unique_key:data.unique_key
				}
			}).then(response=>{
				if(response) callback(null,teacher_details);
				callback({status:0},{status:0,message:"Invalid Reset Password Link!"});
            }).catch(err=>{
                callback({status:1},{status:0,error:err,message:"Error in server. Please try again after some timeome time"});
            })
		}
		//destroy all token by email
		function destroyAllToken(teacher_details,callback) {
			StudentTeacherEmailToken.destroy({ 
				where:{ email:teacher_details.student_teachers_email_address,token_type:"forgot_password"},
			}).then(response=>{
				callback({status:0},{status:1,message:"Token has been verified successfully",teacher_details:teacher_details });
            }).catch(err=>{
                callback({status:1},{status:0,error:err,message:"Error in server. Please try again after some timeome time"});
            })
		}
	})
}
//Reset Password
student_teachers.resetPassword=(data)=>{
	return new Promise((resolve,reject)=>{
        async.waterfall([
			checkTeacherExistence,
			resetPassword
        ], (err, result)=> {
            if(err.status == 1) reject(result);
            else resolve(result);
		});
		//check teacher existence
		function checkTeacherExistence(callback) {
			StudentTeacher.findOne({ 
					where:{ student_teachers_email_address:data.student_teachers_email_address },
					attributes : ['student_teachers_id','student_teachers_first_name','student_teachers_last_name','student_teachers_email_address'],
				}).then(teacher_details=>{
					if(teacher_details) callback(null,teacher_details);
					else callback({status:0},{status:0,message:"Check & enter a valid registered email address!"});
			}).catch(err=>{
				callback({status:1},{status:0,error:err,message:"Error in server. Please try again after some timeome time"});
			})
		}
		//reset password
		function resetPassword(teacher_details,callback) {
			StudentTeacher.update({ 
				student_teachers_password:helpers.encrypt(data.student_teachers_password)
			},{ 
				where:{student_teachers_email_address:teacher_details.student_teachers_email_address} 
			}).then(response=>{
				callback({status:0},{status:1,message:"Password has been reset successfully!"});
            }).catch(err=>{
                callback({status:1},{status:0,error:err,message:"Error in server. Please try again after some timeome time"});
            })
		}

	})
}
//fetch Student List
student_teachers.fetchStudent=(data)=>{
	return new Promise((resolve,reject)=>{
		StudentTeacher.findAll({
			where:{
				student_teachers_email_address:data.student_teachers_email_address,
				student_grade_level:{ $in: data.student_grade_level},
				is_active:1,
				is_deleted:0
			},
			attributes: ['student_teachers_id','student_id','student_grade_level','student_first_name','student_middle_name','student_last_name','student_school_name','student_school_city','student_school_state','student_school_zip_code'],
			distinct: true,
		}).then(response=>{

			var student_list=JSON.parse(JSON.stringify(response));
			/* 	
		    Fall = August 1 – November 30
		    Winter = December 1 – Feb 28/29
		    Spring = March 1 - July 31
		    */
		    var year=moment().year();
		    var today_date = moment(new Date(), "DD/MM/YYYY");
		    var fall_start_date   = moment("01/08/"+year, "DD/MM/YYYY");
		    var fall_End_date     = moment("30/11/"+year, "DD/MM/YYYY");

		    var winter_start_date   = moment("01/12/"+year, "DD/MM/YYYY");
		    var winter_End_date     = moment("28/02/"+year, "DD/MM/YYYY");

		    var spring_start_date   = moment("01/03/"+year, "DD/MM/YYYY");
		    var spring_End_date     = moment("31/07/"+year, "DD/MM/YYYY");

		    if(today_date.isBetween(fall_start_date, fall_End_date)) {
		        var session="Fall";
		    }else if(today_date.isBetween(winter_start_date, winter_End_date)) {
		        var session="Winter";
		    }else if(today_date.isBetween(spring_start_date, spring_End_date)) {
		        var session="Spring";
		    }else{
		        var session="Holiday";
		    }
			resolve({status:1,message:"All Student List",session:session,student_list:student_list});
		}).catch(err=>{
			reject({status:0,error:err,message:"Error in server. Please try after some time"});
		})
	});
}
//sentence list
student_teachers.sentenceList=(data)=>{
	return new Promise((resolve,reject)=>{
		Sentence.findAll({
			where:data,
			attributes: ['sentence_id','sentence','complexity_band'],
		}).then(response=>{
			var sentence_list=JSON.parse(JSON.stringify(response));
			resolve({status:1,message:"All Sentence List",sentence_list:sentence_list});
		}).catch(err=>{
			reject({status:0,error:err,message:"Error in server. Please try after some time"});
		})
	});
}

//Admin Header Detail, forgot password, reset password, change password, edit profile
student_teachers.getUserProfile = function (res, data, next) {
    StudentTeacher.findOne({
        where:data
    }).then(function (result) {
        next(result);
    }).catch(function (err) {
        return res.send({
            status: 0,
            msg: "Database error!"
        });
    });
}


student_teachers.sentOtpMail = function (res, userlist, next) {
	var digits = '0123456789'; 
    let OTP = ''; 
    for (let i = 0; i < 5; i++ ) { 
        OTP += digits[Math.floor(Math.random() * 10)]; 
    } 
	var where={ 
		student_teachers_email_address:userlist.student_teachers_email_address,	
	}

	console.log(OTP);


	StudentTeacher.update({
			one_time_password:OTP,
		},{
		where:where
	}).then(function (result) {
        helpers.readHTMLFile(path.join(__dirname,'../email_templates/one-time-password.html'), (err, html) =>{
			if (err) {
				return res.send({
		            status: 0,
		            msg: "Sorry! We are unable to sent mail to your email address: "+userlist.student_teachers_email_address
		        });
			}else{
				var template = handlebars.compile(html);			
				var replacements = {
					full_name: userlist.student_teachers_first_name+' '+userlist.student_teachers_last_name,
					OTP:OTP,
				};
				var html = template(replacements);
				helpers.sentMail(userlist.student_teachers_email_address,"One Time Password",siteUrl.support_email,html,(response)=>{
					if(response.status==1){
						next({status:1,message:"One time password has been sent to your registered email - "+userlist.student_teachers_email_address});
					}else{
						next({status:0,message:"Sorry! We are unable to sent mail to your email address: "+userlist.student_teachers_email_address});
					}
				});
			}
			 
		})
    }).catch(function (err) {
        return res.send({
            status: 0,
            msg: "Database error!"
        });
    });		
}


student_teachers.checkOldPassword = function (res, otp, password, next) {
	let data = {one_time_password: otp, student_teachers_password:helpers.encrypt(password)}
    StudentTeacher.findAndCountAll({
        where:data
    }).then(function (result) {
        next(result);
    }).catch(function (err) {
        return res.send({
            status: 0,
            msg: "Database error!"
        });
    });
}

student_teachers.updateNewPassword = function (res, otp, password, next) {

	let where = {one_time_password: otp}

	StudentTeacher.update({
		student_teachers_password:helpers.encrypt(password),
		one_time_password:''
	},{
		where:where
	}).then(function (result) {
        next(result);
    }).catch(function (err) {
        return res.send({
            status: 0,
            msg: "Database error!"
        });
    });
}



module.exports = student_teachers;
