/* Importing schemas */
const TestResult = require('./schemas/TestResultSchema');
const StudentTeacher = require('./schemas/StudentTeacherSchema');
/* Importing database*/
const Sequelize = require('sequelize');
const sequelize = require('../config/database').sequelize;
/* importing */
const async=require('async');

var test_results={};

//save test result
test_results.saveTestResult=(data)=>{
	return new Promise((resolve,reject)=>{
		async.waterfall([
            checkStudentExistence,
            checkGiveSummativeTestOrNot,
			saveTestResult,
		],(err, result)=> {
			if(err.status==0) reject(result);
			else resolve(result);
		});
		//check student exist or not using student teachers id and grade level
		function checkStudentExistence(callback){
			StudentTeacher.findOne({
				where:{ 
					student_teachers_id:data.student_teachers_id,	
					student_grade_level:data.student_grade_level 
				},
			}).then(student_teachers_details=>{
				if(student_teachers_details){
					var student_teachers_details=JSON.parse(JSON.stringify(student_teachers_details));
					callback(null,student_teachers_details);
				}else callback({status:1},{status:0,message:"Invalid Student!"});
			}).catch(err=>{
				callback({status:0},{status:0,error:err,message:"Error in server. Please try after some time"});
			})
        }
        //check summative exam give or not
        function checkGiveSummativeTestOrNot(student_teachers_details,callback){
            if(data.test_type=="formative"){
                TestResult.count({
                    where:{ 
                        student_teachers_id:data.student_teachers_id,	
                        student_grade_level:data.student_grade_level,
                        test_type:'summative' 
                    },
                }).then(count=>{
                    if(count>0) callback(null,student_teachers_details);
                    else callback({status:1},{status:0,message:"Student must have previously taken the Letter ID Summative assessment before taking the Formative assessment!"});
                }).catch(err=>{
                    callback({status:0},{status:0,error:err,message:"Error in server. Please try after some time"});
                })
            }else callback(null,student_teachers_details);
          
        }
        //save test result
		function saveTestResult(student_teachers_details,callback){
			data.student_id=student_teachers_details.student_id;
			data.student_teachers_email_address=student_teachers_details.student_teachers_email_address;
			data.student_name=student_teachers_details.student_first_name+" "+student_teachers_details.student_middle_name+" "+student_teachers_details.student_last_name;
			data.student_teachers_name=student_teachers_details.student_teachers_first_name+" "+student_teachers_details.student_teachers_last_name;
			TestResult.create(data).then(response=>{
				callback({status:1},{status:1,message:"Test result has been saved successfully"});
			}).catch(err=>{
				callback({status:0},{status:0,error:err,message:"Error in server. Please try after some time"});
			})
		}
	});
}
//get question bank for formative
test_results.getQuestionBankFormative=(data)=>{
	return new Promise((resolve,reject)=>{
		async.waterfall([
			checkStudentExistence,
			checkGiveSummativeTestOrNot,
            createQuestionBank
		],(err, result)=> {
			if(err.status==0) reject(result);
			else resolve(result);
		});
		//check student exist or not using student teachers id and grade level
		function checkStudentExistence(callback){
			StudentTeacher.count({
				where:{ 
					student_teachers_id:data.student_teachers_id,	
					student_grade_level:data.student_grade_level 
				},
			}).then(count=>{
				if(count>0) callback(null);
				else callback({status:1},{status:0,message:"Invalid Student!"});
			}).catch(err=>{
				callback({status:0},{status:0,error:err,message:"Error in server. Please try after some time"});
			})
		}
		 //check summative exam give or not
		 function checkGiveSummativeTestOrNot(callback){
			TestResult.findOne({
				where:{ 
					student_teachers_id:data.student_teachers_id,	
					student_grade_level:data.student_grade_level,
					test_type:'summative' 
				},
				order: [
					['test_result_id', 'DESC']
				],
				attributes: ['correct_words', 'incorrect_words']
			}).then(response=>{
				if(response) callback(null,response);
				else callback({status:1},{status:0,message:"Student must have previously taken the Letter ID Summative assessment before taking the Formative assessment!"});
			}).catch(err=>{
				callback({status:0},{status:0,error:err,message:"Error in server. Please try after some time"});
			})
		}
		//create question bank
		function createQuestionBank(summative_data,callback){
			var correct_words=JSON.parse(summative_data.correct_words);
			var incorrect_words=JSON.parse(summative_data.incorrect_words);
			var question_banks=[];
			if(correct_words.length>5){
				var correct_question=[];
				var count=5;
				while(count){
					var random_word=get_random(correct_words);
					if(!correct_question.includes(random_word)){
						correct_question.push(random_word);
						count--;
					}
				}
				question_banks = incorrect_words.concat(correct_question);
			}else{
				question_banks = incorrect_words.concat(correct_words);
			}
			callback({status:1},{status:1,message:"Question Bank!",question_banks:question_banks});
		}
	});
}
function get_random(list) {
	return list[Math.floor((Math.random()*list.length))];
} 
module.exports = test_results;
