const express = require('express');
const router = express.Router();
const trimRequest = require('trim-request');
const path=require('path');
/*importing all middleware*/
const authMiddleware = require('../middlewares/authentication');
const CreateUploadPathDirectory=require('../middlewares/CreateUploadPathDirectory');
const FileUpload=require('../middlewares/FileUpload');
/*importing all controllers*/
const student_teachers = require('../controllers/StudentTeacherController');
const test_results = require('../controllers/TestResultController');
const sentence_fluency_test_results = require('../controllers/SentenceFluencyTestResultController');
const fluency_comprehension_summative = require('../controllers/FluencyComprehensionSummativeController');
const fluency_comprehension_formative = require('../controllers/FluencyComprehensionFormativeController');
const spelling_summative_words = require('../controllers/SpellingSummativeWordController');
/* GET index */
router.get('/', function(req, res, next) {
  res.sendFile(path.join(__dirname, '../views', '/index.html'));
});
//Fetch all teachers email through grade level
router.get('/fetch-all-teachers-email',trimRequest.all,student_teachers.fetchAllTeachersEmail);
//Teachers registration
router.post('/teachers-registration',trimRequest.all,student_teachers.teachersRegistration);
//Teacher's login
router.post('/teachers-login',trimRequest.all,student_teachers.teachersLogin);
//logout
router.get('/logout',trimRequest.all,authMiddleware.auth,student_teachers.logout);
//Forgot password
router.post('/forgot-password',trimRequest.all,student_teachers.forgotPassword);
//verify token for forgot password
router.post('/verify-token-forgot-password',trimRequest.all,student_teachers.verifyTokenForgotPassword);
//reset password
router.post('/reset-password',trimRequest.all,student_teachers.resetPassword);
//student list
router.get('/fetch-student',trimRequest.all,authMiddleware.auth,student_teachers.fetchStudent);
//save test result
router.post('/save-test-result',trimRequest.all,authMiddleware.auth,test_results.saveTestResult);
//get question bank for formative
router.post('/get-question-bank-formative',trimRequest.all,authMiddleware.auth,test_results.getQuestionBankFormative);
//sentence list
router.get('/sentence-list',trimRequest.all,authMiddleware.auth,student_teachers.sentenceList);
//Save Sentence Fluency Test Result
router.post('/save-sentence-fluency-test-result',trimRequest.all,authMiddleware.auth,sentence_fluency_test_results.saveSentenceFluencyTestResult);
//get summative paragraphs
router.get('/get-fluency-comprehension-summative-paragraphs',trimRequest.all,authMiddleware.auth,fluency_comprehension_summative.getFluencyComprehensionSummativeParagraphs);
//get summative questions
router.get('/get-fluency-comprehension-summative-questions',trimRequest.all,authMiddleware.auth,fluency_comprehension_summative.getFluencyComprehensionSummativeQuestions);
//Save fluency comprehension Test Result
router.post('/save-fluency-comprehension-test-result',trimRequest.all,authMiddleware.auth,CreateUploadPathDirectory.createUploadPathDirectory('record_audio'),FileUpload.fileUpload,fluency_comprehension_summative.saveFluencyComprehensionTestResult);
//get formative complexity band
router.get('/get-fluency-comprehension-formative-complexity-bands',trimRequest.all,authMiddleware.auth,fluency_comprehension_formative.getFluencyComprehensionFormativeComplexityBands);
//get formative paragraphs
router.get('/get-fluency-comprehension-formative-paragraphs',trimRequest.all,authMiddleware.auth,fluency_comprehension_formative.getFluencyComprehensionFormativeParagraphs);
//get formative questions
router.get('/get-fluency-comprehension-formative-questions',trimRequest.all,authMiddleware.auth,fluency_comprehension_formative.getFluencyComprehensionFormativeQuestions);
//Save fluency comprehension Test Result
router.post('/save-fluency-comprehension-formative-test-result',trimRequest.all,authMiddleware.auth,CreateUploadPathDirectory.createUploadPathDirectory('record_audio'),FileUpload.fileUpload,fluency_comprehension_formative.saveFluencyComprehensionTestResult);
//get spelling summative words
router.get('/get-spelling-summative-words',trimRequest.all,authMiddleware.auth,spelling_summative_words.getSpellingSummativeWord);
//save spelling summative test result
router.post('/save-spelling-summative-test-result',trimRequest.all,authMiddleware.auth,spelling_summative_words.saveSpellingSummativeTestResult);
//get spelling summative words
router.get('/complete-current-spelling-test',trimRequest.all,authMiddleware.auth,spelling_summative_words.completeCurrentSpellingTest);
/* GET index */
router.get('/reset-password/:email/:token', function(req, res, next) {
  res.sendFile(path.join(__dirname, '../views', '/index1.html'));
});

//Forgot password
router.post('/forgot-password-new',trimRequest.all,student_teachers.forgotPasswordNew);

router.post('/reset-password-new',trimRequest.all,student_teachers.resetPasswordNew);

module.exports = router;
